local MemoryVersion = 2;

local L = LibStub("AceLocale-3.0"):GetLocale("Short term");
local LG = LibStub("AceLocale-3.0"):GetLocale("Short term message groups");
local MAJOR = "ShortTerm";
local ShortTerm = LibStub("AceAddon-3.0"):GetAddon(MAJOR);
local LibRingCache = LibStub:GetLibrary("LibRingCache-1.0");

GlobalShortTermMemory = {
	__VERSION__ = MemoryVersion,
	[1] = {
		Data = {},
		Events = {
			BN_WHISPER = true,
			BN_CONVERSATION = true,
		},
		Length = 100,
		Name = L.MSGGROUP_DEFAULT,
	},
};
ShortTermMemory = {
	__VERSION__ = MemoryVersion;
	[1] = {
		Data = {},
		Events = {
			AFK = true,     BATTLEGROUND_LEADER = true,
			SAY = true,     BATTLEGROUND = true,
			EMOTE = true,   GUILD_OFFICER = true,
			YELL = true,    MONSTER_SAY = true,
			WHISPER = true, MONSTER_YELL = true,
			PARTY = true,   MONSTER_EMOTE = true,
			RAID = true,    MONSTER_WHISPER = true,
			GUILD = true,   MONSTER_BOSS_EMOTE = true,
			IGNORED = true, MONSTER_BOSS_WHISPER = true,
			SKILL = true,   RAID_WARNING = true,
			LOOT = true,    RAID_LEADER = true,
			MONEY = true,   BG_NEUTRAL = true,
			BG_HORDE = true,BG_ALLIANCE = true,
			CHANNEL = true, ACHIEVEMENT = true,
			DND = true,     GUILD_ACHIEVEMENT = true,
		},
		Length = 100,
		Name = L.MSGGROUP_DEFAULT,
	},
};
ShortTermMemoryPrototype = {
	--- TODO: Scopes: Guild, ServerFaction, Server
	Name = "Default",
	Scope = "CHARACTER",
	Length = 100,
	Events = {},
	Data = {},
};

ShortTerm.MemoryVersion = MemoryVersion;
ShortTerm.Defaults = {};
ShortTerm.Defaults.Prototype = ShortTermMemoryPrototype;
ShortTerm.Defaults.Global = GlobalShortTermMemory;
ShortTerm.Defaults.Character = ShortTermMemory;
local Events = {
	SYSTEM = true,         BATTLEGROUND_LEADER = true,
	SAY = true,            GUILD = true,
	EMOTE = true,          GUILD_OFFICER = true,
	YELL = true,           MONSTER_SAY = true,
	WHISPER = true,        MONSTER_YELL = true,
	PARTY = true,          MONSTER_EMOTE = true,
	RAID = true,           MONSTER_WHISPER = true,
	RAID_LEADER = true,    MONSTER_BOSS_EMOTE = true,
	RAID_WARNING = true,   MONSTER_BOSS_WHISPER = true,
	BATTLEGROUND = true,   ERRORS = true,
	AFK = true,            SKILL = true,
	DND = true,            LOOT = true,
	IGNORED = true,        MONEY = true,
	BG_HORDE = true,       OPENING = true,
	BG_ALLIANCE = true,    TRADESKILLS = true,
	BG_NEUTRAL = true,     PET_INFO = true,
	COMBAT_XP_GAIN = true, COMBAT_MISC_INFO = true,
	ACHIEVEMENT = true,    COMBAT_HONOR_GAIN = true,
	CHANNEL = true,        COMBAT_FACTION_CHANGE = true,
	GUILD_ACHIEVEMENT = true,
	BN_WHISPER = true,     BN_CONVERSATION = true,
	BN_TOAST = true,
};


--[[ OPTIONS ]]--

local optionstable = {
	type = "group",
	name = L.SHORT_TERM_MEMORY,
	childGroups = "tab",
	args = {
		__com = {
			name = L.DISCLAIMER,
			type = "description",
			order = 2,
		},
		__gui = {
			name = "Config dialog",
			order = 1,
			guiHidden = true,
			dialogHidden = true,
			type = "execute",
			func = function() InterfaceOptionsFrame_OpenToCategory(optionstablecategory); end,
		},
		__general = {
			name = L.GENERAL,
			order = 3,
			type = "group",
			args = {
				addgroup = {
					name = L.ADDGROUP,
					order = 1,
					type = "group",
					disabled = true,
					args = {
						addgroupinput = {
							name = L.ADDGROUP,
							desc = L.ADDGROUP_DESC,
							type = "input",
							set = function(info, val)
								val = tonumber(val);
							end,
							get = function() return "text"; end,
						},
						addgroupbutton = {
							name = L.ADDGROUP,
							desc = L.ADDGROUP_DESC,
							type = "execute",
							func = function() return; end,
						},
					}
				},
				bnwhisperbehaviour = {
					name = L.BNWHISPCONV,
					desc = L.BNWHISPCONV_DESC,
					type = "select",
					values = function()
						return {
							[1] = L.BNWHISP_CONV_NOCHANGE,
							[2] = L.BNWHISP_CONV_TOWHISP,
							[3] = L.BNWHISP_CONV_NICKNAME,
						};
					end,
					get = function(info, index, ...)
						local parent = info[#info-1];
						_G.INSPECTINFO = info;
						_G.MOREARGRUMENTS = {...};
						return GlobalShortTermMemory.__BNWHISPER_MODE or 1;
					end,
					set = function(info, index, val)
						GlobalShortTermMemory.__BNWHISPER_MODE = val;
					end,
				},
				--[[
				showfragments = {
					name = L.SHOW_FRAGMENTS,
					desc = L.SHOW_FRAGMENTS_DESC,
					type = "toggle",
					order = 3,
					set = function(info,val)
						ShortTermMemorySettings.ShowFragments = val;
					end,
					get = function()
						return ShortTermMemorySettings.ShowFragments or false;
					end,
				},
				]]
			},
		},
		-- more groups go here
	},
};

local getcacheinfo(info)
	local option = info.option; -- .option should be added by metatable
	local frame, settings;
	local id = option.stm_id;
	if option.stm_scope == "GLOBAL" then
		frame = ShortTerm.GlobalFrames[id];
		settings = GlobalShortTermMemory[id];
	else
		frame = ShortTerm.Frames[id];
		settings = ShortTermMemory[id];
	end
	
	return frame, settings;
end

local MessageGroup = {
	name = L.TYPES,
	desc = L.TYPES_DESC,
	order = 4,
	type = "group",
	stm_scope = "GLOBAL",
	stm_id = 1,
	args = {
		desc = {
			name = L.TYPES_DESC,
			type = "description",
			order = 1,
		},
		backloglength = {
			name = L.BACKLOG_LENGTH,
			desc = L.BACKLOG_LENGTH_DESC,
			type = "range",
			order = 2,
			min = 10,
			softMax = 500,
			max = 10000,
			step = 5,
			bigStep = 20,
			width = "full",
			set = function(info, val)
				local frame, settings = getcacheinfo(info);
				
				if settings then
					settings.Length = tonumber(val);
					-- for now lets just hope the player does not do this too often.
					frame.cache = LibRingCache:createBlackbox(val, ShortTermMemory);
				end
			end,
			get = function(info)
				local frame, settings = getcacheinfo(info);
				
				if settings then 
					if not settings.Length then
						settings.Length = 50;
					end
					return settings.Length;
				end
			end,
		},
		--[[
		backlogtime = {
			name = L.BACKLOG_TIME,
			desc = L.BACKLOG_TIME_DESC,
			type = "range",
			order = 2,
			disabled = true,
			min = 0,
			max = 72,
			step = 1,
			bigStep = 1,
			width = "full",
			set = function(info,val)
				ShortTermMemorySettings.BacklogTimeOut = val;
			end,
			get = function() return ShortTermMemorySettings.BacklogTimeOut or 2; end,
		},
		]]
		types = {
			name = L.TYPES2,
			type = "multiselect",
			order = 3,
			width = "full",
			values = function()
				local ks = {};
				for k,v in pairs(Events) do
					ks[k] = LG[k];
				end
				return ks;
			end,
			get = function(info, index, ...)
				local parent = info[#info-1];
				_G.INSPECTINFO = info;
				_G.MOREARGRUMENTS = {...};
				local frame, settings = getcacheinfo(info);
				return settings.Events[index] or false;
			end,
			set = function(info, index, val)
				local frame, settings = getcacheinfo(info);
				settings.Events[index] = val or nil;
			end,
		}
	},
};


local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");

local function shallowcopy(tbl)
	local ret = {};
	for k, v in pairs(tbl) do
		ret[k] = v;
	end
	return ret;
end

AceConfig:RegisterOptionsTable("Short term memory", function()
	for k, v in ipairs(ShortTermMemory) do
		local cpy = shallowcopy(MessageGroup);
		--cpy.name = L[k];
		cpy.name = v.Name;
		cpy.stm_scope = v.Scope;
		cpy.stm_id = k;
		--cpy.data =
		optionstable.args["msggroup"..k] = cpy;
	end
	return optionstable;
end);
optionstablecategory = AceConfigDialog:AddToBlizOptions("Short term memory", nil, nil);
